% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spacexr.R
\name{runRctd}
\alias{runRctd}
\title{Run RCTD algorithm to decompose cell type mixtures}
\usage{
runRctd(RCTD, rctd_mode = "doublet")
}
\arguments{
\item{RCTD}{\code{\linkS4class{RctdConfig}} object created using
\code{\link{createRctd}}}

\item{rctd_mode}{character string specifying the RCTD mode:
\code{"doublet"}, \code{"multi"}, or \code{"full"}
(default: \code{"doublet"})}
}
\value{
A \code{\link[SpatialExperiment]{SpatialExperiment}} object
  containing the RCTD results with:
  \itemize{
    \item Three assays (one in full mode):
      \itemize{
        \item \code{weights}: Cell type proportions restricted according to
        the specified mode (not available in full mode)
        \item \code{weights_unconfident}: Cell type proportions restricted
        according to the specified mode, including unconfident predictions
        (not available in full mode)
        \item \code{weights_full}: Unrestricted cell type proportions
      }
    Assays have cell types as rows and pixels as columns, with values
    representing the proportion (0 to 1) of each cell type in each pixel.
    Assay columns sum to 1 (except for rejected pixels, which sum to 0).
    \item \code{spatialCoords} containing spatial coordinates for each pixel
    \item \code{colData} containing:
      \itemize{
        \item For doublet mode:
          \itemize{
            \item \code{spot_class}: Classification as \code{"singlet"},
            \code{"doublet_certain"}, \code{"doublet_uncertain"}, or
            \code{"reject"}
            \item \code{first_type}, \code{second_type}: Predicted cell types
            \item \code{first_class}, \code{second_class}: Whether
            predictions were made at the class level
            \item Additional metrics like \code{min_score},
            \code{singlet_score}
          }
        \item For multi mode:
          \itemize{
            \item \code{cell_type_list}: List of cell types per pixel
            \item \code{conf_list}: List of whether cell type predictions
            are confident
            \item Additional metrics like \code{min_score}
          }
      }
  }
}
\description{
Runs the RCTD algorithm to decompose cell type mixtures in spatial
transcriptomics data. For each pixel in the spatial data, RCTD estimates the
proportions of different cell types by comparing the gene expression at the
pixel to the reference cell type profiles, while accounting for technical
factors like platform effects and varying sequencing depth. The algorithm has
three modes suited for different spatial technologies:
\itemize{
  \item \code{doublet}: Fits at most two cell types per pixel and classifies
    each pixel as a \code{"singlet"} (one cell type) or \code{"doublet"}
    (two cell types). Best for high spatial resolution technologies like
    Slide-seq or MERFISH, where pixels are more likely to contain only 1 or
    2 cells.
  \item \code{multi}: Uses a greedy algorithm to fit up to
    \code{MAX_MULTI_TYPES} cell types per pixel (default: 4). Best for lower
    resolution technologies like 100-micron Visium spots, which can contain
    more cell types.
  \item \code{full}: Fits any number of cell types per pixel without
    restrictions.
}
}
\examples{
data(rctdSim)

# Spatial transcriptomics data
library(SpatialExperiment)
spatial_spe <- SpatialExperiment(
    assay = rctdSim$spatial_rna_counts,
    spatialCoords = rctdSim$spatial_rna_coords
)

# Reference data
library(SummarizedExperiment)
reference_se <- SummarizedExperiment(
    assays = list(counts = rctdSim$reference_counts),
    colData = rctdSim$reference_cell_types
)

# Create RCTD configuration
rctd_config <- createRctd(spatial_spe, reference_se, max_cores = 1)
results <- runRctd(rctd_config, rctd_mode = "doublet")

# Access the cell type proportions (cell types as rows, pixels as columns)
assay(results, "weights")

# Check spot classifications for doublet mode
colData(results)$spot_class

# Access spatial coordinates
head(spatialCoords(results))

}
